﻿using System.ComponentModel;
using System.Runtime.InteropServices;

namespace Dapfor.Wpf.Grid.Benchmark.Utils
{
    internal class HighPerformanceTimer
    {
        private readonly long _freq;
        private long _startTime;
        private long _stopTime;

        // Constructor
        public HighPerformanceTimer()
        {
            _startTime = 0;
            _stopTime = 0;

            if (QueryPerformanceFrequency(out _freq) == false)
            {
                // high-performance counter not supported
                throw new Win32Exception();
            }
        }

        // Returns the duration of the timer (in milliseconds)
        public double Duration
        {
            get { return (double) 1000*(_stopTime - _startTime)/_freq; }
        }

        [DllImport("Kernel32.dll")]
        private static extern bool QueryPerformanceCounter(
            out long lpPerformanceCount);

        [DllImport("Kernel32.dll")]
        private static extern bool QueryPerformanceFrequency(
            out long lpFrequency);

        // Start the timer
        public void Start()
        {
            QueryPerformanceCounter(out _startTime);
        }

        // Stop the timer
        public void Stop()
        {
            QueryPerformanceCounter(out _stopTime);
        }
    }
}